/*--------------------------------
 * 
 * Jerome.Gouzy@toulouse.inra.fr
 * 
 * $Id: til-r.h 539 2017-08-18 15:05:10Z gouzy $
 * 
 ---------------------------------*/


#ifndef TILR_H

#define TILR_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "math.h"
#include "paramparser.h"
#include "assert.h"
#include "generalbioinfo.h"
#include "general.h"
#include "fasta_utils.h"
#include "uthash.h"
#include "utarray.h"

#define DEFAULT_PARAM_NCPUS	8
#define MAX_THREAD_LOCKS    20000
#define BUFFER_SIZE 		2048
#define MINWINGLEN          0 // pour essayer d'eviter les hits avec un ccs
#define DEFAULT_PARAM_MINWING   	 1000
#define DEFAULT_PARAM_MINSEEDENDCOV  3
#define DEFAULT_PARAM_MINOVL  		 1000
#define DEFAULT_PARAM_MINSEQLEN  	 3000
#define DEFAULT_PARAM_MINPCI  		 97.0
#define DEFAULT_PARAM_DELTAPCI  	 3.0
#define DEFAULT_PARAM_PENALTYFACT  	 1.0
#define DEFAULT_PARAM_SPEMODE	  	 1
#define DEFAULT_PARAM_SLICEX		 1.0
#define ST_INIT     0
#define ST_AVOID    -1
#define ST_SEED     1
#define ST_KEEP     2
#define SELECTED_OVL     1
#define ALL_OVL     2

#define ANCHOR_INIT   1
#define ANCHOR_LOADED 2

//#define MINSUMCOV   20

typedef struct ovlwith
{
	unsigned int seqid; // id avec sequence avec qui il y a overlap
	int          len;   // longueur de l'overlap (sur la ref)
	int          wing;  // longueur de la nouvelle region
	int          qstart; // start of the overlap on the query (added to compute anchors)
	short        tstrand;
	short        qstrand;
	short        pci;   // int(valeur en float*100)
	short        selected; // est ce que l'overlap est selectionne
	int          score;
} ovlwith_t;

typedef struct region
{
	int  start;
	int  end; 
} region_t;

typedef struct seq
{
	UT_array*   ovl5;  // tableau ovlwith_t 5
	UT_array*   ovl3;  // tableau ovlwith_t 3
	short       maxpci3; // 100*pci
	short       maxpci5;
	int         maxwing3; // extension possible
	int         maxwing5;
	int         contig;
	int         seqlen;
	int         len;      // longueur de l'overlap (sur la ref)
	short       status;   // est ce que la sequence doit etre evite ?
	short       seedable; // est ce que la sequence peut etre utilise comme graine (deux ext couvertes ~ pareil
	short       slice_id; 
} seq_t;

typedef struct seqsortportance
{
	int seqid;
	int portance;
} seqsortportance_t;

void InitDataStruct(seq_t **a_seq,UT_array** a_spe_regions,unsigned int maxid,int mode);

void Usage (char *argv[]);
void GetMeanCov(seq_t **a_seq,int maxid,double *meancov,int *maxcov,int print);

unsigned long long GetGenomeSize(char *buffsize);

int  SortPortanceDecreasing (void const *a, void const *b);
void InitPortance(seqsortportance_t **a_seqportance,int nbseqok,seq_t **a_seq,int maxid);
int SortOnDecreasingOvlLen(const void *a,const void*b);
int SortOnDecreasingPortance(const void *a,const void*b);

int  CompressListOfOverlaps(seq_t **a_seq,int maxid,int selection);
void CompressListOfOverlapsOneEnd(seq_t **a_seq,UT_array *ovl,int selection);

//=====================
// Nettoyage du graphe
//=====================
void IterativeCheckOfWings(seq_t **a_seq,int maxid, int minwing);
int  KeepSequencesAboveMinwing(seq_t **a_seq,int maxid,int minwing);
void InitWings(seq_t **a_seq,int maxid,int minwing);
int  InitWingsOneEnd(seq_t **a_seq,UT_array *ovl);

// removing multiple overlaps between the same pair
void TagRedundantOverlaps(seq_t **a_seq,int maxid);
void TagRedundantOverlapsOneEnd(seq_t **a_seq,UT_array *ovl);

// filtering non reciprocal hits
void CleanupNoReciprocal(seq_t **a_seq,int maxid);
void CleanupNoReciprocalOneEnd(seq_t **a_seq,UT_array *ovl,unsigned long long seqid);
int  IsReciprocal(seq_t *p_seqrecip,unsigned long long seqid);
int  HasOverlapWithOneEnd(UT_array *ovl,int seqid);
void SelectReciprocalOverlap(seq_t *p_seq,int seqid);

// pci based filtering
void CleanupBasedOnIdentity(seq_t **a_seq,int maxid,float minpci, float deltapcif);
void CleanupBasedOnIdentityOneEnd(seq_t **a_seq,UT_array *ovl,short int maxpci,short deltapci,unsigned long long seqid);
int  CheckReciprocalIdentity(seq_t *p_seqrecip,int seqid,ovlwith_t *p_ovl,short deltapci);
int  CheckReciprocalIdentityOneEnd(UT_array *ovl,short pci,short deltapci,unsigned long long seqid,short maxpci);

// remove secondary links not linked to the best one (largest overlap)
int  ValidateOverlaps(seq_t **a_seq,int maxid,seqsortportance_t **a_seqportance,int nbseqok);
void ValidateOverlapsOneEnd(seq_t **a_seq,UT_array *ovl,int seqid,char str[250]);

// remove spurious double links (3 and 5' link to the same target)
int  RemoveDoubleLinked(seq_t **a_seq,int maxid);

//=================================
// Read/Dump falcon formatted files
//=================================
void LoadOverlaps(char *ovlin,seq_t **a_seq,float minpci,int minovl,int minseqlen,UT_array **a_spe_regions,int speregionsloaded,int spemode);
void DumpOverlaps(char *ovlin,seq_t **a_seq,char *ovlout,float minpci,int minovl,float deltapcif,int mode);
void DumpOverlapsOneEnd(FILE *f_out,char *line,seq_t **a_seq,ovlwith_t *read_ovl,int target,UT_array *ovl,int mode);
void RemoveContained(char *ovlcontainer,seq_t **a_seq,unsigned long long maxid,int minwing);

//=============================================
// Read/Dump/Compute speicfic regions (anchors)
//=============================================
void DumpAnchorRegions(char *out,UT_array **a_spe_regions,unsigned long long maxid);
void LoadAnchorRegions(char *speregions,UT_array **a_spe_regions);
int  FindAnchor(UT_array **a_spe_regions,int query,int start,int end);
void ComputeAnchorRegions(sequence_t **a_sequences,unsigned long long nb_seq,seq_t **a_seq,unsigned long long maxid,UT_array **a_spe_regions,unsigned long long genomesize,char *ovlin,float slice_fact);



#endif

/*
void CleanupNotConfirmed(seq_t **a_seq,int maxid);
int SelectReciprocal(seq_t *p_seqrecip,int seqid,ovlwith_t *p_ovl);
int  SortOnCovDecreasing(const void *a,const void*b);
int  SelectOverlaps(seq_t **a_seq,int maxid,seqsortportance_t **a_seqportance,int nbseqok);
void SortTrimOnMean(seq_t **a_seq,int maxid,short cutoff,float penalty_factor);
int  SortOnIncreasingOvl(const void *a,const void*b);
int OverWingSelection(seq_t **a_seq,int maxid,seqsortportance_t **a_seqportance,int nbseqok,int minwing);
void DumpSeqOk(seq_t **a_seq,int maxid,seqsortportance_t **a_seqportance,int nbseqok,int minseedendcov);
int SeedSelection(seq_t **a_seq,int maxid,seqsortportance_t **a_seqportance,int nbseqok,int minseedendcov);
int BridgeSelection(seq_t **a_seq,int maxid,seqsortportance_t **a_seqportance,int nbseqok);
void CheckSeedEnds(seq_t **a_seq,int maxid,seqsortportance_t **a_seqportance,int nbseqok);
void PrintSeqOvl(seq_t **a_seq,int seqid);
void PrintOvl(int end,int cpt,ovlwith_t *p_ovl);
int CleanOverlaps(seq_t **a_seq,int maxid,seqsortportance_t **a_seqportance,int nbseqok);
*/
